#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "graph.h"
#include "cube.h"

word seed,
     col,
     lin,
     deep,
     ext,
     prob;

word num_edges;


void init_rand (void) {

 srand (seed);

}


int rand_n (void) {

  return (rand () % 100 + 1);

}

int rand_lin (void) {

  return ((rand () % lin) + 1);

}

int rand_col (void) {

  return ((rand () % col) + 1);

}

int rand_deep (void) {

  return ((rand () % deep) + 1);

}

void test_rand_n (void) {

  int i,
      j = 0,
      k = 0,
      l;

  for (i = 0; i < 30000; i++) {

    l = rand_n ();

    assert (l >= 1 && l <= 100);

    if (l <= 50)

      j++;

    else

      k++;

  }

  /*printf ("%d,%d\n", j, k);*/

  j = k = 0;

  for (i = 0; i < 30000; i++) {

    l = rand_lin ();


    assert (l >= 1 && l <= lin);


    if (l <= lin / 2)

      j++;

    else

      k++;

  }

  /*printf ("line %d,%d\n", j, k);*/

  j = k = 0;


  for (i = 0; i < 30000; i++) {

    l = rand_col ();

    assert (l >= 1 || l <= col);


    if (l <= col / 2)

      j++;

    else

      k++;

  }

  /*printf ("column %d,%d\n", j, k);*/

}


void read_config (void) {

  FILE *config;

  if ( (config = fopen (config_file, "r")) == NULL) {

    printf ("Couldn't open configuration\n");
    exit (0);

  }

  fscanf (config, "seed %d\n", &seed);
  fscanf (config, "columns %d\n", &col);
  fscanf (config, "lines %d\n", &lin);
  fscanf (config, "deep %d\n", &deep);
  fscanf (config, "extra %d\n", &ext);
  fscanf (config, "prob %d\n", &prob);

  /*printf ("seed %d\n", seed);
  printf ("columns %d\n", col);
  printf ("lines %d\n", lin);
  printf ("deep %d\n", deep);
  printf ("extra %d\n", ext);
  printf ("prob %d\n", prob);*/


  fclose (config);

}


void build_random_net (graph *graph_ptr) {

  int d, i, j;


  graph_ptr->num_vertices = lin * col * deep + ext;

  /* array of vertices, vertices [1..n] / n = |V| */

  graph_ptr->vert_ptr = ALLOC (graph_ptr->num_vertices + 1, vertex);

  /* array of mates, mate [1..n] / n = |V| */

  graph_ptr->mate_ptr = ALLOC (graph_ptr->num_vertices + 1, mate);

  for (d = 1; d <= deep; d++)

    for (i = 1; i <= lin; i++)

      for (j = 1; j <= col; j++) {

	if (j != col)

	  if (rand_n () <= prob) {

	      assert ( (d - 1)*(lin*col) + (i-1)*col+j >= 1 && (d - 1)*(lin*col) +  (i-1)*col+j <= graph_ptr->num_vertices);

	      assert ( (d - 1)*(lin*col) + (i-1)*col+j+1 >= 1 &&  (d - 1)*(lin*col) + (i-1)*col+j+1 <= graph_ptr->num_vertices);

	      add_edge (graph_ptr, (d - 1)*(lin*col) +  (i-1)*col+j, (d - 1)*(lin*col) +  (i-1)*col+j+1);

	    }

	if (i != lin)

	  if (rand_n () <= prob) {

	    assert ( (d - 1)*(lin*col) + (i-1)*col+j >= 1 &&  (d - 1)*(lin*col) + (i-1)*col+j <= graph_ptr->num_vertices);

	    assert ( (d - 1)*(lin*col) + i*col+j >= 1 && (d - 1)*(lin*col) +  i*col+j <= graph_ptr->num_vertices);

	    add_edge (graph_ptr, (d - 1)*(lin*col) +  (i-1)*col+j, (d - 1)*(lin*col) +  (i)*col+j);

	  }


	if (d != deep)

	  if (rand_n () <= prob) {

	    assert ( (d - 1)*(lin*col) + (i-1)*col+j >= 1 &&  (d - 1)*(lin*col) + (i-1)*col+j <= graph_ptr->num_vertices);

	    assert ( (d)*(lin*col) + (i-1)*col+j >= 1 &&  (d)*(lin*col) + (i-1)*col+j <= graph_ptr->num_vertices);

	    add_edge (graph_ptr, (d - 1)*(lin*col) + (i-1)*col+j,  d*(lin*col) + (i-1)*col+j);

	  }


      }

  for (i = 1; i <= graph_ptr->num_vertices; i++) {

    int v;


    if (graph_ptr->vert_ptr [i].edge_ptr == NULL) {

      do {

	v = (rand_deep () - 1)*(lin*col) + (rand_lin () - 1) * col + rand_col ();

	assert (v >= 1 && v <= graph_ptr->num_vertices);

      } while (v == i);

       add_edge (graph_ptr, i, v);
    }


  }

}


void save_graph (graph *graph_ptr, word edges) {


  FILE *output;

  int i;


  assert ((output = fopen (output_file, "w")) != NULL);

  fprintf (output, "c Gerador de grafos em forma de grade\n");
  fprintf (output, "s %d\n", seed);
  fprintf (output, "p %d %d\n", graph_ptr->num_vertices, edges);



  for (i = 1; i <= graph_ptr->num_vertices; i++) {

    edge *edge_ptr = graph_ptr->vert_ptr [i].edge_ptr;

    while (edge_ptr) {

      if (!edge_ptr->visited) {

	edge *edge_aux = graph_ptr->vert_ptr [edge_ptr->vert].edge_ptr;

	fprintf (output, "a %d %d\n", i, edge_ptr->vert);

	edge_ptr->visited = TRUE;

	while (edge_aux->vert != i)

	  edge_aux = edge_aux->next;

	edge_aux->visited = TRUE;

      }

      edge_ptr = edge_ptr->next;

    }



  }


  fclose (output);



}

void main (int argc, char *argv []) {

  graph *graph_ptr = ALLOC (1, graph);

  word edges;



if (argc != 7) { printf ("cube seed cols lins deep extra prob\n"); exit (0); }
/*  read_config ();*/

   seed = atoi (argv [1]);
   col = atoi (argv [2]);
   lin = atoi (argv [3]);
   deep = atoi (argv [4]);
   ext = atoi (argv [5]);
   prob = atoi (argv [6]);

  test_rand_n ();

  init_rand ();

  build_random_net (graph_ptr);

  maximal_matching (graph_ptr);

  edges = statistic_graph (graph_ptr);

  save_graph (graph_ptr, edges);

  free_graph (graph_ptr);

}
