/***********\
 * graph.h *
\***********/

/*======================*\
|* Optional definitions *|
\*======================*/


/*
#define SPECIFIC_INPUT      force the input to read files that specify
			    matched and unmatched edges
			    format of the imput file:
				p num_vert num_edges
				s seed
				c comentary
				a i j 0|1 (0 = free, 1 = matched) */


/*=====================*\
|* General definitions *|
\*=====================*/


#define MAX_THREADS 20  /* Maximum number of threads,
			   may be chanded freely        */
#define TRUE  1
#define FALSE 0

#define NORMAL  0       /* function exit: normal exit   */
#define ERROR  -1       /* function exit: abnormal exit */

#define ALLOC(X,Y) (Y*)calloc(X,sizeof(Y)) /* ALLOC (itens, type) */


#define MATCHED(X) graph_ptr->mate_ptr [graph_ptr->mate_ptr [X].mate].mate == X

#define VERT_FREE(X) graph_ptr->mate_ptr [graph_ptr->mate_ptr [X].mate].mate != X


/*=======================*\
|* Algorithm definitions *|
\*=======================*/


/* Manipulate the flag of a vertex. */

  /* questions */

#define IS_VISITED(X)     (X&1)
#define IS_EVEN(X)        (X&2)
#define IS_ODD(X)         (!(X&2))
#define IS_RIGHT(X)       (X&4)
#define IS_OUTER(X)       (X&8)
#define IS_BFS_ACTIVE(X)  (X&16)

  /* actions */

#define UNVISITED(X)      (X=0)
#define VISITED(X)        (X=X|1)
#define EVEN(X)           (X=X|2)
#define ODD(X)            (X=X&253)
#define RIGHT(X)          (X=X|4)
#define LEFT(X)           (X=X&251)
#define OUTER(X)          (X=X|8)
#define INNER(X)          (X=X&247)
#define BFS_ACTIVE(X)     (X=X|16)
#define BFS_INACTIVE(X)   (X=X&239)


/*==================*\
|* type definitions *|
\*==================*/


typedef unsigned int word;

typedef int boole;

  /* Edge node of the adjacency list */

typedef struct edge_node {

  word vert;                 /* adjacent vertex */

  boole visited;

  struct edge_node *next;    /* implement the list of edges */

} edge;


  /* vertex array */

typedef struct vertex_node {

  boole root;               /* flag indicating that the vertex  was a  root
			       of a search. If it was then is not necessary
			       to make a search again from this  vertex. */
  edge *edge_ptr;           /* edges of the vertex */

} vertex;


/* used to make a list of free vertices */

typedef struct vert_free_node {

  word vert;

  struct vert_free_node *next;

} free_vert;


  /* mate array. If vertex u is mate of v
     then mate(u) = v and mate (v) = u */

typedef struct mate_node {

  word mate;

} mate;


  /* level array. Indicate the level of the vertex
     in the bread-first search */

typedef struct level_node {

  word level;

} level;


  /* predecessor array. Pointer to the father of a
     vertex in the tree formed by the  bread-first
     search */

typedef struct pred_node {

  word pred;

} pred;


  /* blossom list. store the basic informatios of
     the blossoms */

typedef struct blossom_node {

  word left_peak, right_peak; /* left and right peaks */

  word base;                  /* base of the blossom  */

  struct blossom_node *next;  /* inplement  a  list.  */

} blossom;

  /* blossom array. Each vertex has a pointer to a blossom
     if it belongs to one. Obs: One vertex can  belong  to
     only one blossom. */

typedef struct bl_node {

  blossom *bl;

} bl;

  /* flag array for eache vertex. Each bit corresponds to
     an information.

      first  bit: 1 if the vertex is visited 2 unvisited
      second bit: 1 if the vertex is even 2 if odd
      third  bit: 1 if the vertex is right 2 if left
      forth  bit: 1 if the vertex is outer 2 if inner
      fifth  bit: 1 if the vertex is active 2 inactive */

typedef struct flag_node {

  char flag;

} flag;


  /* bread_first search array. Circular array that
     stores the vertex at the limit of the  search */

typedef struct bfs_node {

  word vert;

} bfs;


  /* list that stores the vertex that comprises the
     blossom */

typedef struct bl_create_node {

  word vert;

  struct bl_create_node *next;  /* implement a list */

} bl_create;


  /* struct that stores the basic information to do
     a search localy at each thread */

typedef struct search_node {

  word size;                   /* size of the bread-first
				  search cirular list     */
  word begin;                  /* pointer to the beginnig
				  of the informations at
				  the circular list       */
  word end;                    /* pointer to the end of
				  the iformations at the
				  circular list           */
  level *level_ptr;            /* level of each vertex    */

  pred *pred_ptr;              /* predecessors array      */

  flag *flag_ptr;              /* flag array              */

  blossom* blossom_ptr;        /* list of blossoms formed
				  during the  actual
				  search                  */
  bl *bl_ptr;                  /* array of pointer to the
				  blossoms                */
  bl_create *bl_create_ptr;    /* temporary list to store
				  the vertices that will
				  form a new blossom      */
  bfs *bfs_ptr;                /* circular list of BFS    */

} search;


  /* struct commom to all threads */

typedef struct graph_node {

  word num_vertices;         /* number of the vertices
				of the graph           */
  vertex *vert_ptr;          /* array of vertices      */

  free_vert *free_vert_ptr;  /* list of free vertices  */

  mate *mate_ptr;            /* matching array         */

} graph;


  /* used to ordenate the vertices by cadinality */

typedef struct {

  word order,    /* index of the vertex */

       card;     /* level of the vertex */

} list_vert;


void add_edge (graph *, word, word);

boole there_is_edge (graph *, word, word);

void maximal_matching (graph *);

word statistic_graph (graph *);

void free_graph (graph *);









