/*************************************************************************\
 * library.c - library of functions to manipulate adjaceny list of graph *
\*************************************************************************/

#include "graph.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <limits.h>

extern word lin, col;


int NUM_PROCESSORS;

char input_file_name [256];

char output_file_name [256];

extern word match_init;


/* parse_command_line - get the parameters from the command line
   input - the number of paramenters and the respective strings 
   obs - fill the global variables NUM_PROCESSORS, input_file_name
         output_file_name */

void parse_command_line (int argc, char *argv []) {

  if (argc == 1 || argc > 4) {

    printf ("USAGE: prlmatch input CPUs [output]\n");
    exit (NORMAL);

  }

  strcpy (input_file_name, argv [1]);

  NUM_PROCESSORS = atoi (argv [2]);

  if (NUM_PROCESSORS < 1 || NUM_PROCESSORS > MAX_THREADS) {

    printf ("USAGE: prlmatch input CPUs [output]\n");
    exit (NORMAL);
  
  }

  if (argc == 4)

    strcpy (output_file_name, argv [3]);

  else

    strcpy (output_file_name, "");

}


/* add_edge - insert an edge into the graph structure
   input    - graph, vertices i and j, (1 <= i,j <= n) */

void add_edge (graph *graph_ptr, word i, word j) {

  edge *edge_ptr;

  vertex *vert_ptr;


  /* create the edges */

  edge_ptr = ALLOC (1, edge);


  vert_ptr = &graph_ptr->vert_ptr [i];
  edge_ptr->vert = j;
  edge_ptr->next = vert_ptr->edge_ptr;
  vert_ptr->edge_ptr = edge_ptr;


  edge_ptr = ALLOC (1, edge);

  vert_ptr = &graph_ptr->vert_ptr [j];
  edge_ptr->vert = i;
  edge_ptr->next = vert_ptr->edge_ptr;
  vert_ptr->edge_ptr = edge_ptr;

}



boole there_is_edge (graph *graph_ptr, word i, word j) {

  edge *edge_ptr = graph_ptr->vert_ptr [i].edge_ptr;

  boole res = FALSE;


  while (edge_ptr) {

    if (edge_ptr->vert == j)

      res = TRUE;

    edge_ptr = edge_ptr->next;

  }

  return res;

}


/* maximal_matching - produces an initial matching non ordered manner
   input - the graph structure */

void maximal_matching (graph *graph_ptr) {

  word count;


  for (count = 1; count <= graph_ptr->num_vertices; count++) {

    if (VERT_FREE (count)) {

      edge *edge_ptr = graph_ptr->vert_ptr [count].edge_ptr;


      while (edge_ptr) {

        int adj = edge_ptr->vert;

        if (VERT_FREE (adj)) {

           graph_ptr->mate_ptr [count].mate = adj;
           graph_ptr->mate_ptr [adj].mate = count;

           break;
        }

        else 

          edge_ptr = edge_ptr->next;

      }

    }

  }

}


/* print_graph - print the adjacency list
   input - the graph structure */

void print_graph (graph *graph_ptr) {

  word count;

  for (count = 1; count <= graph_ptr->num_vertices; count++) {

    edge *edge_ptr = graph_ptr->vert_ptr [count].edge_ptr;


    printf ("(%d)", count);

    while (edge_ptr != NULL) {

      printf (" %d", edge_ptr->vert);

      edge_ptr = edge_ptr->next;

    }

    putchar ('\n');

  }

}


/* statisctic_graph - inform the number of vertices matched 
   input - the graph structure */

word statistic_graph (graph *graph_ptr) {

  word vert,

       vert_alone = 0,

       edges = 0,

       vert_free = 0,

       matched_edges = 0,

       match = 0;


  for (vert = 1; vert <= graph_ptr->num_vertices; vert++) {

    vertex *vert_ptr = &graph_ptr->vert_ptr [vert];

    edge *edge_ptr = vert_ptr->edge_ptr;

    if (VERT_FREE (vert))

      vert_free++;


    if (vert_ptr->edge_ptr == NULL)

      vert_alone++;

    while (edge_ptr != NULL) {

      edges++;

      if (graph_ptr->mate_ptr [vert].mate == edge_ptr->vert &&
	  graph_ptr->mate_ptr [edge_ptr->vert].mate == vert)

	matched_edges++;

      edge_ptr = edge_ptr->next;

    }

    if (MATCHED (vert))

      match++;

  }

  /*printf ("Vertices %d\n", graph_ptr->num_vertices);

  printf ("Vertices alones %d\n", vert_alone);

  printf ("Complete net edges %d\n", (col - 1) * lin + (lin - 1) * col);

  printf ("Edges %d\n", edges / 2);

  printf ("Free vertices %d\n", vert_free);

  printf ("Matched edges %d\n", matched_edges / 2);

  printf ("%% Free vertices %d\n", 100 * (vert_free - vert_alone) / (graph_ptr->num_vertices - vert_alone));
*/
  return edges / 2;

}




/* free_graph - free all memory used by the program
   input - the graph structure */

void free_graph (graph *graph_ptr) {

  word count;


  for (count = 1; count <= graph_ptr->num_vertices; count++) {

    vertex *vert_ptr = &graph_ptr->vert_ptr [count];


    while (vert_ptr->edge_ptr != NULL) {

      edge *edge_ptr = vert_ptr->edge_ptr;


      vert_ptr->edge_ptr = vert_ptr->edge_ptr->next;

      free (edge_ptr);

    }

  }


  free (graph_ptr->vert_ptr);

  free (graph_ptr->mate_ptr);

  free (graph_ptr);

}


/* compare_card - function to compare elements into the list of levels 
   input - two pointers to elements of the list
   output - (-1,0,1) depending on the level of the vertex */

int compare_card (const void *elem_a, const void *elem_b) {

  list_vert *list_vert_ptr_a = (list_vert *) elem_a,
            *list_vert_ptr_b = (list_vert *) elem_b;


  if (list_vert_ptr_a->card < list_vert_ptr_b->card)

    return (-1);

  if (list_vert_ptr_a->card > list_vert_ptr_b->card)

    return (1);

  return (0);

}


/* vertex_level - inform the level of a vertex 
   input - the graph structure and the vertex */

word vertex_level (graph *graph_ptr, word vert) {

  edge *edge_ptr = graph_ptr->vert_ptr [vert].edge_ptr;

  word count = 0;


  while (edge_ptr) {

    count++;

    edge_ptr = edge_ptr->next;

  }

  return (count);

}


/* original_order - produce a list which indicate the order
                    that each thread must visit the vertices.
                    this function produce an order 1,2,...,n 
   input - the graph structure */

list_vert *original_order (graph *graph_ptr) {

  word vert,

       num_vert = graph_ptr->num_vertices;

  list_vert *list_vert_ptr = ALLOC (num_vert + 1, list_vert);


  /* fill the list with the vertices informations */

  for (vert = 1; vert <= num_vert; vert++) {

    list_vert_ptr [vert].card = 0;

    list_vert_ptr [vert].order = vert;

  }
 
  return (list_vert_ptr);

}


/* sort_by_cardinality - sort the vertices by cardinality
   input - the graph structure 
   output - the array of vertices ordered */

list_vert *sort_by_cardinality (graph *graph_ptr) {

  word vert,

       num_vert = graph_ptr->num_vertices;

  list_vert *list_vert_ptr = ALLOC (num_vert + 1, list_vert);


  /* fill the list with the vertices informations */

  list_vert_ptr [0].order = 0, list_vert_ptr [0].card = 0;

  for (vert = 1; vert <= num_vert; vert++) {

    list_vert_ptr [vert].card = vertex_level (graph_ptr, vert);

    list_vert_ptr [vert].order = vert;

  }

  qsort ((void *) list_vert_ptr, num_vert + 1,
         sizeof (list_vert), compare_card);

  return (list_vert_ptr);
   
}

