/* graph.h */


/* Optional definitions */

#define DOS /* UNIX */

/*
#define SPECIFIC_INPUT     /* force the input to read files that specify
			      matched and unmatched edges
			      format of the imput file:
			      p num_vert num_edges
			      s seed
			      c comentary
			      a i j 0|1 (0 = free, 1 = matched) */


/* General definitions */

#define TRUE  1
#define FALSE 0

#define NORMAL  0 /* used in exit functions, normal exit */
#define ERROR  -1 /*          ,,             abnormal exit */

#define ALLOC(X,Y) (Y*)alloc_mem(X,sizeof(Y)) /* ALLOC (itens, type) */


/* General type definitions */

typedef unsigned int word;

typedef unsigned char boolean;

typedef enum {  	      /* mark left by the double depth first search */

    NONE,                     /* unmarked */

    LEFT,                     /* vertex belong to the left_DFS */

    RIGHT                     /* vertex belogn to the right_DFS */

} DFS_Tree;


/* Edge node of the adjacency list */

typedef struct edge_node {

  word vertex;                 /* adjacent vertex */

  boolean used;                /* TRUE => edge is used */

  boolean visited;             /* TRUE => edge is visited */

  boolean matched;             /* TRUE => edge is matched */

  struct edge_node *pred_next; /* vertex is a predecessor
				  of the owner of the edge */

  struct edge_node *succ_next; /* vertex is a successor of the owner
				  of the edge */

  struct edge_node *anom_next; /* vertex is an anomaly of the
				  owner of the edge */

  struct edge_node *mate;      /* edge <i, j> => mate <j, i> */

  struct edge_node *next;      /* implemente the list of edges */

} edge;


/* vertex array */

typedef struct vertex_node {

  word even_level, odd_level;      /* levels of the vertex, UINT_MAX is
				      infinity */
  word bloom_number;               /* bloom's identification */

  word base;                       /* base of the bloom */

  word count;                      /* number of unerased predecessors */

  word left_parent;                /* used to find the left path of
				      a bloom */
  word right_parent;               /* used to find the right path of
				      a bloom */
  word vert_bloom;                 /* used to find the path through blooms */

  boolean erased;                  /* TRUE => vertex is erased */

  boolean visited;                 /* TRUE => vertex is visited */

  DFS_Tree mark;	           /* mark left by the double depth
				      first search */
  edge *edge_list;                 /* edges of the vertex */

  edge *pred_list;                 /* implement the predecessors list */

  edge *succ_list;                 /* implement the successors list */

  edge *anom_list;                 /* implement the anomalies list */

} vertex;


/* bloom list */

typedef struct bloom_node {

  word bloom_number;             /* bloom's identification */

  word vertex_left_peak;         /* left peak */

  word vertex_right_peak;        /* right peak */

  struct bloom_node *next;       /* implement the list of blooms */

} bloom;


/* bloom creation */

typedef struct bloom_creation_node {

  word vertex_num;                  /* vertex that may belong to a bloom */

  struct bloom_creation_node *next; /* implementation of a list */

} bloom_creation;


/* task to do */

typedef struct task_node {

  enum {        /* type of the task to be done */

    BFS,        /* part of a bread-first search. vert_1 holds the vertex */

    BRIDGE,     /* a bridge must be treated. vert_1 and
		   vert_2 are the peaks */
    FINDPATH    /* paths must be found. vert_1=high and
		   vert_2=low of path_1 */
  } type;

  word vert_1, vert_2, vert_3, vert_4;

  enum {        /* status of the path_1 and path_2 */

    TO_BE_DONE,

    BEING_DONE,

    DONE

  } path_1, path_2;

  struct task_node *next; /* implement the task list */

} task;


/* find structure */

typedef struct find_node {

  word vert_num;                /* vertex visited by the procedure
				   find_path */
  struct find_node *next, *ant; /* implementation of a double-linked list */

} find;

typedef struct point_node {

  word x,y;

} point;


/* main structure */

typedef struct graph_node {

  word num_vertices;      /* number of the vertices of the graph */

  point *point_ptr;


  vertex *vert_array;     /* array of vertices */

  bloom *bloom_list;      /* list of the blooms */

  task **task_ptr_array;  /* array of pointers to tasks */

} graph;


/* used to ordenate the vertices by cadinality */

typedef struct {

  word order,    /* index of the vertex */

       card;     /* cardinality of the vertex */

} list_vert;


/* io.c prototypes */

graph *input_graph (void);

void output_graph (graph *);


/* library.c prototypes */

void parse_command_line (int, char * []);

void *alloc_mem (word, word);

void start_time (void);

void end_time (void);

void maximal_matching_1 (graph *);

word cardinality (graph *, word);

int compare_card (const void *, const void *);

void maximal_matching_2 (graph *);

void add_edge (graph *, word, word);

void add_predecessor (vertex *, edge *);

void add_successor (vertex *, edge *);

void add_anomaly (vertex *, edge *);

void set_predecessor (vertex *, edge *);

void set_successor (vertex *, edge *);

void set_anomaly (vertex *, edge *);

word base (graph *, word, DFS_Tree);

void add_bloom (graph *, word, word, word);

void get_bloom_peaks (graph *, word, word *, word *);

bloom_creation *add_vertex_to_bloom (bloom_creation *, word);

void create_bloom (graph *, bloom_creation *, word, word);

void free_creation_bloom_list (bloom_creation *);

void add_task (graph *, word, task *);

void verify_match (graph *);

void clear_visited_marks (graph *);

boolean search (graph *, word, word);

void verify_maximum_match (graph *);

void print_graph (graph *);

void statistic_graph (graph *);

void free_graph (graph *);


/* matching.c prototypes */

void start_aug_info (void);

void next_phase_aug (void);

boolean actual_phase_aug (void);

boolean last_phase_aug (void);

void set_actual_phase_aug (void);

void initialization (graph *);

word level (graph *, word);

boolean exposed (graph *, word);

boolean initialize_tasks (graph *);

boolean there_is_tasks (graph *);

task *get_task (graph *);

void even_BFS (graph *, word);

void odd_BFS (graph *, word);

boolean left_DFS (graph *, word, word *, word *, word *, bloom_creation **,
		  boolean *);

void right_DFS (graph *, word *, word *, word *, word *,
		bloom_creation **, boolean *);

void reverse_path (find **, find **);

void open_path (graph *, word, find **, find **);

void find_path (graph *, word, word, word, DFS_Tree,find **, find **);

void invert_edge (graph *, word, word);

void invert_path (graph *graph_ptr, find *);

void erase_successors (graph *, word);

void erase_path (graph *, find *);

void free_path (find *);

void set_other_level (graph *, word);

void blossom_augmentation (graph *, word, word);

void task_manager (graph *);

void general_matching_algorithm (graph *);
